package org.apache.ojb.odmg;

import org.odmg.Transaction;

/* Copyright 2003-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
public interface TransactionExt extends Transaction, HasBroker
{
    /**
     * marks an object for deletion without
     * locking the object.
     *
     * @param  anObject Object to be marked
     */
    public void markDelete(Object anObject);

    /**
     * marks an object as dirty without
     * locking the object.
     *
     * @param  anObject Object to be marked
     */
    public void markDirty(Object anObject);

    /**
     * <p>
     * Calling <code>flush</code> flushes persistent object modifications
     * made within the ODMG transaction since the last checkpoint to the underlying
     * database transaction, but does <b<not</b> commit the database transaction.
     * The ODMG transaction retains all locks it held on those objects at the time the flush
     * was invoked.
     * <p/>
     * This method is very similair to {@link org.odmg.Transaction#checkpoint}.
     */
    public void flush();
    
    
    /**
     * This method can be used to activate or deactivate the implicit
     * locking mechanism for the current transaction.
     * turning of implicit locking may improve performance but requires
     * additional care to make sure all changed objects are properly
     * registered to the transaction.
     * @param value if set to true implicit locking is enabled, 
     *        if false, implicit locking is disabled.
     **/
    public void setImplicitLocking(boolean value);
}
